@@ -40,6 +40,7 @@
 #include "eog-transform.h"
 #include "eog-util.h"
 #include "eog-jobs.h"
+#include "eog-thumbnail.h"
 
 #include <unistd.h>
 
@@ -616,6 +617,13 @@ eog_image_apply_display_profile (EogImage *img, cmsHPROFILE screen)
 
 	if (screen == NULL || priv->profile == NULL) return;
 
+	/* TODO: support other colorspaces than RGB */
+	if (cmsGetColorSpace (priv->profile) != icSigRgbData ||
+	    cmsGetColorSpace (screen) != icSigRgbData) {
+		eog_debug_message (DEBUG_LCMS, "One or both ICC profiles not in RGB colorspace; not correcting");
+		return;
+	}
+
 	transform = cmsCreateTransform (priv->profile, 
 				        TYPE_RGB_8, 
 				        screen, 
@@ -623,17 +631,18 @@ eog_image_apply_display_profile (EogImage *img, cmsHPROFILE screen)
 				        INTENT_PERCEPTUAL, 
 				        0);
 	
-	rows = gdk_pixbuf_get_height(priv->image);
-	width = gdk_pixbuf_get_width (priv->image);
-	stride = gdk_pixbuf_get_rowstride (priv->image);
-	p = gdk_pixbuf_get_pixels (priv->image);
+	if (G_LIKELY (transform != NULL)) {
+		rows = gdk_pixbuf_get_height (priv->image);
+		width = gdk_pixbuf_get_width (priv->image);
+		stride = gdk_pixbuf_get_rowstride (priv->image);
+		p = gdk_pixbuf_get_pixels (priv->image);
 
-	for (row = 0; row < rows; ++row) {
-		cmsDoTransform(transform, p, p, width);
-		p += stride;
+		for (row = 0; row < rows; ++row) {
+			cmsDoTransform (transform, p, p, width);
+			p += stride;
+		}
+		cmsDeleteTransform (transform);
 	}
-
-	cmsDeleteTransform (transform);
 }
 
 static void
@@ -666,8 +675,6 @@ eog_image_set_icc_data (EogImage *img, EogMetadataReader *md_reader)
 		return;
 	}
 
-	return;
-
 #ifdef HAVE_EXIF
 	if (priv->exif == NULL) return;
 
@@ -900,24 +907,15 @@ eog_image_get_dimension_from_thumbnail (EogImage *image,
 			                gint     *width,
 			                gint     *height)
 {
-	const char *w, *h;
-
 	if (image->priv->thumbnail == NULL)
 		return FALSE;
-	
-	w = gdk_pixbuf_get_option (image->priv->thumbnail,
-				   "tEXt::Thumb::Image::Width");
 
-	h = gdk_pixbuf_get_option (image->priv->thumbnail,
-				   "tEXt::Thumb::Image::Height");
-	
-	if (w)
-		sscanf (w, "%i", width);
+	*width = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (image->priv->thumbnail),
+						     EOG_THUMBNAIL_ORIGINAL_WIDTH));
+	*height = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (image->priv->thumbnail),
+						      EOG_THUMBNAIL_ORIGINAL_HEIGHT));
 
-	if (h)
-		sscanf (h, "%i", height);
-	
-	return (w && h);
+	return (*width || *height);
 }
 
 static gboolean
@@ -956,7 +954,7 @@ eog_image_real_load (EogImage *img,
 
 	eog_image_get_file_info (img, &priv->bytes, &mime_type, error);
 
-	if (*error) {
+	if (error && *error) {
 		g_free (mime_type);
 		return FALSE;
 	}
@@ -998,7 +996,7 @@ eog_image_real_load (EogImage *img,
 
 		loader = gdk_pixbuf_loader_new_with_mime_type (mime_type, error);
 	
-		if (*error) {
+		if (error && *error) {
 			g_error_free (*error);
 			*error = NULL;
 
@@ -1128,7 +1126,7 @@ eog_image_real_load (EogImage *img,
 	failed = (failed ||
 		  priv->cancel_loading || 
 		  bytes_read_total == 0 || 
-		  *error != NULL);
+		  (error && *error != NULL));
 
 	if (failed) {
 		if (priv->cancel_loading) {
@@ -1179,7 +1177,7 @@ eog_image_real_load (EogImage *img,
 	}	
 
 	/* Catch-all in case of poor-error reporting */
-	if (failed && *error == NULL) {
+	if (failed && error && *error == NULL) {
 		g_set_error (error, 
 			     EOG_IMAGE_ERROR,
 			     EOG_IMAGE_ERROR_GENERIC,
@@ -1190,7 +1188,7 @@ eog_image_real_load (EogImage *img,
 }
 
 gboolean
-eog_image_has_data (EogImage *img, guint req_data)
+eog_image_has_data (EogImage *img, EogImageData req_data)
 {
 	EogImagePrivate *priv;
 	gboolean has_data = TRUE;
@@ -1234,7 +1232,7 @@ eog_image_has_data (EogImage *img, guint req_data)
 }
 
 gboolean
-eog_image_load (EogImage *img, guint data2read, EogJob *job, GError **error)
+eog_image_load (EogImage *img, EogImageData data2read, EogJob *job, GError **error)
 {
 	EogImagePrivate *priv;
 	gboolean success = FALSE;
@@ -1250,8 +1248,7 @@ eog_image_load (EogImage *img, guint data2read, EogJob *job, GError **error)
 	}
 
 	if (eog_image_has_data (img, data2read)) {
-		g_warning ("Image %s has requested data already loaded.\n", 
-			    eog_image_get_caption (img));
+		return TRUE;
 	}
 
 	priv->status = EOG_IMAGE_STATUS_LOADING;
@@ -2113,7 +2110,7 @@ compare_quarks (gconstpointer a, gconstpointer b)
 }
 
 GList *
-eog_image_get_supported_mime_types ()
+eog_image_get_supported_mime_types (void)
 {
 	GSList *format_list, *it;
 	gchar **mime_types;
